/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include <winbase.h>

namespace plexdata
{
	namespace critical
	{
		class CCritical
		{
		private:

			CRITICAL_SECTION m_sCritical;

		public:

			CCritical()
			{
				::InitializeCriticalSection( &this->m_sCritical );
			};

			virtual ~CCritical()
			{
				::DeleteCriticalSection( &this->m_sCritical );
			};

			inline void Lock()
			{
				::EnterCriticalSection( &this->m_sCritical );
			};

			inline void Unlock()
			{
				::LeaveCriticalSection( &this->m_sCritical );
			}

#if ( _WIN32_WINNT >= 0x0400 )
#pragma message ( "plexdata::critical::CCritical::TryLock(): Function enabled!" )
			inline bool TryLock()
			{
				return ( ::TryEnterCriticalSection( &this->m_sCritical ) ? true : false );
			}
#endif // _WIN32_WINNT >= 0x0400
		};

		class CAutoLock
		{
		private:

			CCritical& m_cLock;

		public:

			CAutoLock( CCritical* pLock ) 
				: m_cLock( *pLock )
			{
				m_cLock.Lock();
			}

			CAutoLock( CCritical& cLock ) 
				: m_cLock( cLock )
			{
				m_cLock.Lock();
			}

			virtual ~CAutoLock()
			{
				m_cLock.Unlock();
			}
		};
	}
}

// EOF