/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "stdafx.h"

using namespace plexdata::string;

bool ValidateOutputHandle( LPLOGHANDLE pLogger )
{
	// IMPORTANT: Very bad solution to provoke an exception but the only way at the moment!
	__try
	{
		// Access test function and provoke an 
		// exception to ensure pointer validity!
		pLogger->pOutput->Test();
		return true;
	}
	__except( EXCEPTION_EXECUTE_HANDLER  )
	{
#ifdef _DEBUG
#ifdef UNICODE
		_CrtDbgReportW( _CRT_ERROR, _CRT_WIDE(__FILE__), __LINE__, _CRT_WIDE("LogAPI"), _CRT_WIDE("Invalid logger handle!") );
#else
		_CrtDbgReport( _CRT_ERROR, __FILE__, __LINE__, "LogAPI", "Invalid logger handle!" );
#endif // UNICODE
#endif // _DEBUG
		return false;
	}
}

bool ConvertString
	( LPCSTR   szString
	, CString& strResult 
	)
{
	bool success = false;

	if ( szString != NULL )
	{
		size_t length  = strlen( szString ) + 1;
		TCHAR* pBuffer = strResult.CreateBuffer( length );

		if ( pBuffer != NULL )
		{
			int written = ::MultiByteToWideChar
				( CP_ACP
				, 0
				, szString
				, -1
				, pBuffer
				, (int)length
				); 

			if ( written > 0 )
			{
				success = true;
				strResult.CopyBuffer();
			}
			strResult.ReleaseBuffer();
		}
	}
	return success;
}

bool ConvertHandle
	( HANDLE       hLogger
	, LPLOGHANDLE& pLogger 
	)
{
	bool success = false;
	
	pLogger = NULL; // Initialize return value.

	if ( hLogger != NULL && hLogger != INVALID_HANDLE_VALUE )
	{
		pLogger = reinterpret_cast<LPLOGHANDLE>( hLogger );
		
		if ( pLogger != NULL )
		{
			success = ValidateOutputHandle( pLogger );
		}
	}

	return success;
}

bool GetTimestamp
	( CString& strResult 
	)
{
	bool       success = false;
	int        written = -1L;
	long       count   = 25;
	TCHAR*     pBuffer = strResult.CreateBuffer( count );
	SYSTEMTIME theTime = {0};

	if ( pBuffer != NULL )
	{
		::GetLocalTime( &theTime );

		written = swprintf
			( pBuffer
			, count
			, _T("%4.4d-%2.2d-%2.2d %2.2d:%2.2d:%2.2d.%4.4d")
			, theTime.wYear
			, theTime.wMonth
			, theTime.wDay
			, theTime.wHour
			, theTime.wMinute
			, theTime.wSecond
			, theTime.wMilliseconds
			);

		if ( written != -1L )
		{
			strResult.CopyBuffer();
			success = true;
		}

		strResult.ReleaseBuffer();
	}

	return success;
}

bool ExpandSpecialCharacters
	( LPCTSTR  szInput
	, CString& strResult 
	)
{
	bool success = false;

	strResult = _T(""); // Ensure empty return value.

	if ( szInput != NULL && szInput[0] != _T('\0') )
	{
		// Use double size for the buffer to ensure replacement of all characters!
		TCHAR* szTemp = strResult.CreateBuffer( _tcslen( szInput ) * 2 + 1 );

		if ( NULL != szTemp )
		{
			size_t offset  = 0;
			TCHAR  percent = _T('%');

			for ( size_t index=0; szInput[index] != _T('\0'); index++ )
			{
				// Add an additional percent character to output string 
				// if current character is a percent-sign character.
				if ( percent == szInput[index] )
				{
					szTemp[offset++] = percent;
				}

				// Always copy current character into temp buffer.
				szTemp[offset++] = szInput[index];
			}
			// Fit result value and return successful.
			strResult.CopyBuffer();
			strResult.ReleaseBuffer();
			success = true;
		}
	}

	return success;
}

void PrepareOutput
	( CString&    strResult
	, LPLOGHANDLE pLogger
	, LPCTSTR     szOutput
	, LPCTSTR     szLabel 
	)
{
	// Only to ensure...
	_ASSERT( pLogger  != NULL );
	_ASSERT( szOutput != NULL );
	_ASSERT( szLabel  != NULL );

	strResult = _T(""); // Initialize return value.

	// Suppress all intenionally empty messages!
	if ( szOutput != NULL && szOutput[0] != _T('\0') )
	{
		if ( pLogger->bTimestamp )
		{
			CString timeStamp;
			if ( GetTimestamp( timeStamp ) )
			{
				strResult.Format( _T("%s: "), timeStamp.String() );
			}
		}

		if ( pLogger->bEnableLabel )
		{
			strResult.Append( szLabel );
		}

		CString strOuput;
		// Fails only in case of low memory!
		if ( ExpandSpecialCharacters( szOutput, strOuput ) )
		{
			strResult.Append( strOuput );
		}

		if ( !strResult.IsEmpty() )
		{
			if ( _T('\n') != strResult.GetAt( strResult.Length() - 2 ) )
			{
				strResult.Append( _T("\n") );
			}
		}
	}
}

// EOF