/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "stdafx.h"

using namespace plexdata::logging;
using namespace plexdata::string;
using namespace plexdata::critical;

static LPCWSTR MESSAGE_LABEL = _T("MESSAGE: ");
static LPCWSTR WARNING_LABEL = _T("WARNING: ");
static LPCWSTR ERROR_LABEL   = _T("ERROR:   ");

LOGAPIBASE
__out
HANDLE
LOGAPI
LogAttachA
	( __in DWORD  dwLogType
	, __in BOOL   bTimestamp
	, __in BOOL   bEnableLabel
	, __in BOOL   bOverwrite
	, __in LPCSTR lpName
    )
{
	HANDLE  hResult = NULL;
	CString theName;

	if ( ConvertString( lpName, theName ) )
	{
		hResult = ::LogAttachW
			( dwLogType
			, bTimestamp
			, bEnableLabel
			, bOverwrite
			, theName.String()
			);
	}
	else
	{
		::SetLastError( ERROR_INVALID_PARAMETER );
	}

	return hResult;
}

LOGAPIBASE
__out
HANDLE
LOGAPI
LogAttachW
	( __in DWORD   dwLogType
	, __in BOOL    bTimestamp
	, __in BOOL    bEnableLabel
	, __in BOOL    bOverwrite
	, __in LPCWSTR lpName
    )
{
	LPLOGHANDLE pResult = NULL;

	CAutoLock autolock( g_pCritical );

	if ( lpName != NULL )
	{
		pResult = new LOGHANDLE;

		if ( pResult != NULL )
		{
			pResult->bTimestamp   = bTimestamp   ? true : false;
			pResult->bEnableLabel = bEnableLabel ? true : false;

			switch( dwLogType )
			{
			case LOGTYPE_CONSOLE:
				{
					CConsole* pOutput = new CConsole();

					// TODO: Put more initialization stuff here.

					pResult->pOutput = pOutput;
				}
				break;

			case LOGTYPE_FILE:
				{
					CLogfile* pOutput = new CLogfile();

					// TODO: Put more initialization stuff here.

					pResult->pOutput = pOutput;
				}
				break;

			case LOGTYPE_EVENTLOG:
				{
					CEventlog* pOutput = new CEventlog();

					if ( pOutput != NULL )
					{
						pOutput->MessageResourceBinary( g_hModule );
					}

					pResult->pOutput = pOutput;
				}
				break;

			default:
				{
					delete pResult;
					pResult = NULL;
				}
				break;
			}

			if ( pResult != NULL )
			{
				if ( pResult->pOutput != NULL )
				{
					if ( !pResult->pOutput->Attach( lpName, bOverwrite ? true : false ) )
					{
						delete pResult->pOutput;
						delete pResult;
						pResult = NULL;
					}
					else
					{
						::SetLastError( ERROR_OPEN_FAILED );
					}
				}
				else
				{
					delete pResult;
					pResult = NULL;

					::SetLastError( ERROR_NOT_ENOUGH_MEMORY );
				}
			}
			else
			{
				::SetLastError( ERROR_INVALID_PARAMETER );
			}
		}
		else
		{
			::SetLastError( ERROR_NOT_ENOUGH_MEMORY );
		}
	}
	else
	{
		::SetLastError( ERROR_INVALID_PARAMETER );
	}

	return reinterpret_cast<HANDLE>( pResult );
}


LOGAPIBASE
__out
BOOL
LOGAPI
LogMessageA
	( __in HANDLE hLogger
	, __in LPCSTR lpMessage
    )
{
	BOOL    bResult = FALSE;
	CString theMessage;

	if ( ConvertString( lpMessage, theMessage ) )
	{
		bResult = ::LogMessageW
			( hLogger
			, theMessage.String()
			);
	}
	else
	{
		::SetLastError( ERROR_INVALID_PARAMETER );
	}
	
	return bResult;
}

LOGAPIBASE
__out
BOOL
LOGAPI
LogMessageW
	( __in HANDLE  hLogger
	, __in LPCWSTR lpMessage
    )
{
	BOOL        bResult = FALSE;
	LPLOGHANDLE pLogger = NULL;

	CAutoLock autolock( g_pCritical );

	if ( ConvertHandle( hLogger, pLogger ) )
	{
		CString theMessage;

		PrepareOutput( theMessage, pLogger, lpMessage, MESSAGE_LABEL );

		if ( pLogger->pOutput->Message( theMessage.String() ) )
		{
			bResult = TRUE;
		}
		else
		{
			::SetLastError( ERROR_FUNCTION_FAILED );
		}
	}
	else
	{
		::SetLastError( ERROR_INVALID_PARAMETER );
	}

	return bResult;
}

LOGAPIBASE
__out
BOOL
LOGAPI
LogWarningA
	( __in HANDLE hLogger
	, __in LPCSTR lpWarning
    )
{
	BOOL    bResult = FALSE;
	CString theWarning;

	if ( ConvertString( lpWarning, theWarning ) )
	{
		bResult = ::LogWarningW
			( hLogger
			, theWarning.String()
			);
	}
	else
	{
		::SetLastError( ERROR_INVALID_PARAMETER );
	}

	return bResult;
}

LOGAPIBASE
__out
BOOL
LOGAPI
LogWarningW
	( __in HANDLE  hLogger
	, __in LPCWSTR lpWarning
    )
{
	BOOL        bResult = FALSE;
	LPLOGHANDLE pLogger = NULL;

	CAutoLock autolock( g_pCritical );

	if ( ConvertHandle( hLogger, pLogger ) )
	{
		CString theWarning;

		PrepareOutput( theWarning, pLogger, lpWarning, WARNING_LABEL );

		if ( pLogger->pOutput->Warning( theWarning.String() ) )
		{
			bResult = TRUE;
		}
		else
		{
			::SetLastError( ERROR_FUNCTION_FAILED );
		}
	}
	else
	{
		::SetLastError( ERROR_INVALID_PARAMETER );
	}

	return bResult;
}

LOGAPIBASE
__out
BOOL
LOGAPI
LogErrorA
	( __in HANDLE hLogger
	, __in LPCSTR lpError
    )
{
	BOOL    bResult = FALSE;
	CString theError;

	if ( ConvertString( lpError, theError ) )
	{
		bResult = ::LogErrorW
			( hLogger
			, theError.String()
			);
	}
	else
	{
		::SetLastError( ERROR_INVALID_PARAMETER );
	}

	return bResult;
}

LOGAPIBASE
__out
BOOL
LOGAPI
LogErrorW
	( __in HANDLE  hLogger
	, __in LPCWSTR lpError
    )
{
	BOOL        bResult = FALSE;
	LPLOGHANDLE pLogger = NULL;

	CAutoLock autolock( g_pCritical );

	if ( ConvertHandle( hLogger, pLogger ) )
	{
		CString theError;

		PrepareOutput( theError, pLogger, lpError, ERROR_LABEL );

		if ( pLogger->pOutput->Error( theError.String() ) )
		{
			bResult = TRUE;
		}
		else
		{
			::SetLastError( ERROR_FUNCTION_FAILED );
		}
	}
	else
	{
		::SetLastError( ERROR_INVALID_PARAMETER );
	}

	return bResult;
}

LOGAPIBASE
__out
BOOL
LOGAPI
LogDump
	( __in HANDLE hLogger
	, __in LPVOID lpBuffer
	, __in DWORD  dwBuffer
	, __in BOOL   bAbsolute
    )
{
	BOOL        bResult = FALSE;
	LPLOGHANDLE pLogger = NULL;

	CAutoLock autolock( g_pCritical );

	if ( ConvertHandle( hLogger, pLogger ) )
	{
		if ( pLogger->pOutput->Dump( lpBuffer, dwBuffer, bAbsolute ? true : false ) )
		{
			bResult = TRUE;
		}
		else
		{
			::SetLastError( ERROR_FUNCTION_FAILED );
		}
	}
	else
	{
		::SetLastError( ERROR_INVALID_PARAMETER );
	}

	return bResult;
}

LOGAPIBASE
__out
BOOL
LOGAPI
LogResize
	( __in HANDLE hLogger
	, __in DWORD  dwLength
	, __in DWORD  dwWidth
    )
{
	BOOL        bResult = FALSE;
	LPLOGHANDLE pLogger = NULL;

	CAutoLock autolock( g_pCritical );

	if ( ConvertHandle( hLogger, pLogger ) )
	{
		if ( pLogger->pOutput->Resize( dwLength, dwWidth ) )
		{
			bResult = TRUE;
		}
		else
		{
			::SetLastError( ERROR_FUNCTION_FAILED );
		}
	}
	else
	{
		::SetLastError( ERROR_INVALID_PARAMETER );
	}

	return bResult;
}

LOGAPIBASE
__out
BOOL
LOGAPI
LogColors
	( __in HANDLE hLogger
	, __in BOOL   bEnable
    )
{
	BOOL        bResult = FALSE;
	LPLOGHANDLE pLogger = NULL;

	CAutoLock autolock( g_pCritical );

	if ( ConvertHandle( hLogger, pLogger ) )
	{
		pLogger->pOutput->Colors( bEnable ? true : false );
		bResult = TRUE;
	}
	else
	{
		::SetLastError( ERROR_INVALID_PARAMETER );
	}

	return bResult;
}

LOGAPIBASE
VOID
LOGAPI
LogDetach
	( __in HANDLE hLogger
    )
{
	CAutoLock autolock( g_pCritical );

	if ( hLogger != NULL && hLogger != INVALID_HANDLE_VALUE )
	{
		LPLOGHANDLE pLogger = reinterpret_cast<LPLOGHANDLE>( hLogger );

		if ( pLogger != NULL )
		{
			if ( pLogger->pOutput != NULL )
			{
				pLogger->pOutput->Detach();
				delete pLogger->pOutput;
				pLogger->pOutput = NULL;
			}

			delete pLogger;
			pLogger = NULL;
		}
	}

	return;
}

// EOF