/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include <windows.h>
#include <tchar.h>

namespace plexdata
{
	namespace logging
	{
		///////////////////////////////////////////////////////////////////////////
		//
		///////////////////////////////////////////////////////////////////////////
		class COutput
		{
		public:

			COutput();

			virtual ~COutput();

		public:

			virtual bool Attach( LPCTSTR name, bool overwrite ) = 0;

			virtual bool Message( LPCTSTR format, ... ) = 0;

			virtual bool Warning( LPCTSTR format, ... ) = 0;

			virtual bool Error( LPCTSTR format, ... ) = 0;

			virtual bool Dump( const void* pBuffer, unsigned long size, bool absolute ) = 0;

			virtual bool Detach() = 0;

			virtual bool Resize( long length, long width ) = 0;

			virtual void Colors( bool enable ) = 0;

			virtual void Test() = 0;

		public:

			virtual bool Name( LPCTSTR name );

		public:

			inline LPCTSTR Name()   { return this->m_szName;  };
			inline long    Length() { return this->m_nLength; };
			inline long    Width()  { return this->m_nWidth;  };
			inline bool    Colors() { return this->m_bColors; };

		protected:

			bool Extract( LPTSTR& pBuffer, long& count, LPCTSTR pFormat, va_list argList );

			bool PrepareDump( LPTSTR& pOutput, const void* pBuffer, unsigned long size, bool absolute );

		protected:

			LPTSTR m_szName;
			long   m_nLength;
			long   m_nWidth;
			bool   m_bColors;
		};

		///////////////////////////////////////////////////////////////////////////
		//
		///////////////////////////////////////////////////////////////////////////
		class CConsole : public COutput
		{
		public:

			CConsole();

			virtual ~CConsole();

		public:

			bool Attach( LPCTSTR title, bool overwrite=false );

			bool Message( LPCTSTR format, ... );

			bool Warning( LPCTSTR format, ... );

			bool Error( LPCTSTR format, ... );

			bool Dump( const void* pBuffer, unsigned long size, bool absolute=true );

			bool Detach();

			bool Resize( long lines, long columns );

			void Colors( bool enable );

		public:

			inline void Test(){} // Check function to ensure instance validity.
		};

		///////////////////////////////////////////////////////////////////////////
		//
		///////////////////////////////////////////////////////////////////////////
		class CLogfile : public COutput
		{
		public:

			CLogfile();

			virtual ~CLogfile();

		public:

			bool Attach( LPCTSTR file, bool overwrite );

			bool Message( LPCTSTR format, ... );

			bool Warning( LPCTSTR format, ... );

			bool Error( LPCTSTR format, ... );

			bool Dump( const void* pBuffer, unsigned long size, bool absolute=true );

			bool Detach();

			bool Resize( long nSize, long dontCare=0 );

			void Colors( bool dontCare=false );

		public:

			inline void Test(){} // Check function to ensure instance validity.

		private:

			void Convert( LPTSTR& pBuffer, size_t count, void*& pOutput, size_t& length );

		private:

			HANDLE m_hFile;
		};

		///////////////////////////////////////////////////////////////////////////
		//
		///////////////////////////////////////////////////////////////////////////
		class CEventlog : public COutput
		{
		public:

			CEventlog();

			virtual ~CEventlog();

		public:

			bool Attach( LPCTSTR source, bool overwrite=false );

			bool Message( LPCTSTR format, ... );

			bool Warning( LPCTSTR format, ... );

			bool Error( LPCTSTR format, ... );

			bool Dump( const void* pBuffer, unsigned long size, bool absolute=true );

			bool Detach();

			bool Resize( long lengthDonCare=0, long widthDontCare=0 );

			void Colors( bool dontCare=false );

		public:

			inline void Test(){} // Check function to ensure instance validity.

		public:

			bool MessageResourceBinary( HMODULE hResourceBinary );

			bool MessageResourceBinary( LPCTSTR resourceBinary );

		private:

			bool RegisterEventSourceRegistry();

			bool UnregisterEventSourceRegistry();

		private:

			static const LPTSTR REGPATH;
			HANDLE       m_hSource;
			LPTSTR       m_resourceBinary;
		};
	}
}