/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "stdafx.h"
#include "string.h"
#include <stdarg.h>

using namespace plexdata::string;

CString::CString()
	: m_nString(0)
	, m_szString(NULL)
	, m_szBuffer(NULL)
{
	this->Copy( _T("") );
}

CString::CString( const TCHAR* szString )
	: m_nString(0)
	, m_szString(NULL)
	, m_szBuffer(NULL)
{
	this->Copy( szString );
}

CString::CString( CString& refString )
	: m_nString(0)
	, m_szString(NULL)
	, m_szBuffer(NULL)
{
	this->Copy( refString );
}

CString::CString( CString* pString )
	: m_nString(0)
	, m_szString(NULL)
	, m_szBuffer(NULL)
{
	this->Copy( pString );
}

CString::~CString()
{
	this->Destroy();
	this->ReleaseBuffer();
}

bool CString::Append( const TCHAR* szString )
{
	if ( szString != NULL )
	{
		CString strTemp = *this;
		return this->Format( _T("%s%s"), strTemp.m_szString, szString );
	}
	else
	{
		return false;
	}
}

bool CString::Append( CString& refString )
{
	CString strTemp = *this;
	return this->Format( _T("%s%s"), strTemp.m_szString, refString.m_szString );
}

bool CString::Append( CString* pString )
{
	if ( pString != NULL )
	{
		CString strTemp = *this;
		return this->Format( _T("%s%s"), strTemp.m_szString, pString->m_szString );
	}
	else
	{
		return false;
	}
}

bool CString::Copy( const TCHAR* szString )
{
	return this->Create( szString );
}

bool CString::Copy( CString& refString )
{
	return this->Create( refString.m_szString );
}

bool CString::Copy( CString* pString )
{
	if ( pString != NULL )
	{
		return this->Create( pString->m_szString );
	}
	else
	{
		return false;
	}
}

bool CString::Equals( const TCHAR* szString, bool ignore/*=true*/ )
{
	if ( szString != NULL )
	{
		if ( ignore )
		{
			return ( 0 == _tcsicmp( this->m_szString, szString ) );
		}
		else
		{
			return ( 0 == _tcscmp( this->m_szString, szString ) );
		}
	}
	else
	{
		return false;
	}
}

bool CString::Equals( CString& refString, bool ignore/*=true*/ )
{
	if ( refString.m_szString != NULL )
	{
		if ( ignore )
		{
			return ( 0 == _tcsicmp( this->m_szString, refString.m_szString ) );
		}
		else
		{
			return ( 0 == _tcscmp( this->m_szString, refString.m_szString ) );
		}
	}
	else
	{
		return false;
	}
}

bool CString::Equals( CString* pString, bool ignore/*=true*/ )
{
	if ( pString != NULL && pString->m_szString != NULL )
	{
		if ( ignore )
		{
			return ( 0 == _tcsicmp( this->m_szString, pString->m_szString ) );
		}
		else
		{
			return ( 0 == _tcscmp( this->m_szString, pString->m_szString ) );
		}
	}
	else
	{
		return false;
	}
}

bool CString::Format( const TCHAR* szFormat, ... )
{
	bool    success = false;
	size_t  length  = 0;
	double  dummy   = 0.0; // Load floating point library, otherwise runtime error R6002 will occur!
	va_list argList;

	if ( szFormat != NULL )
	{
		va_start( argList, szFormat );

		 // _vsctprintf doesn't include terminating '\0'!
		length = _vsctprintf( szFormat, argList ) + 1;

		if ( length > 0 && this->Create( length ) )
		{
			 // _vstprintf_s doesn't include terminating '\0'!
			success = ( 0 < _vstprintf_s( this->m_szString, this->m_nString, szFormat, argList ) );
		}

		va_end( argList );
	}

	return success;
}

CString CString::Substring( size_t first, size_t last/*=-1L*/ )
{
	CString result;

	if ( this->m_szString != NULL )
	{
		if ( last == -1L )
		{
			last = this->m_nString; // Sub-string is up to the end.
		}
		else if ( last > this->m_nString )
		{
			last = this->m_nString; // Limit result string if needed.
		}

		if ( first <= last )
		{
			size_t count = last - first;

			if ( first < this->m_nString && ( first + count ) <= this->m_nString )
			{
				if ( result.Create( count ) )
				{
					_tcsncpy_s( result.m_szString, result.m_nString + 1, &this->m_szString[first], count );
				}
			}
		}
	}
	return result;
}

CString CString::Trim()
{
	return this->LeftTrim().RightTrim();
}

CString CString::LeftTrim()
{
	CString result;

	if ( this->m_szString != NULL )
	{
		for ( size_t index=0; index < this->m_nString; index++ )
		{
			TCHAR current = this->m_szString[index];
			if ( current != _T(' ') && current != _T('\t') )
			{
				result = this->Substring( index );
				break;
			}
		}
	}
	return result;
}

CString CString::RightTrim()
{
	CString result;

	if ( this->m_szString != NULL )
	{
		// Try to (reverse) find last character.
		for ( size_t index=this->m_nString; index > 0; index-- )
		{
			TCHAR current = this->m_szString[index];

			if ( current != _T(' ') && current != _T('\t') && current != _T('\0') )
			{
				result = this->Substring( 0, index + 1 );
				break;
			}
		}
	}
	return result;
}

size_t CString::FindFirst( const TCHAR find )
{
	size_t result = -1L;

	if ( this->m_szString != NULL )
	{
		// Try to find first character.
		for ( size_t index=0; index < this->m_nString; index++ )
		{
			if ( this->m_szString[index] == find )
			{
				result = index;
				break; // End FOR loop!
			}
		}
	}
	return result;
}

size_t CString::FindNext( const TCHAR find, size_t start )
{
	bool   success = false;
	size_t next    = -1L;

	if ( this->m_szString != NULL && 0 <= start && start < this->m_nString )
	{
		// Try to find next character just behind given position.
		for ( size_t index=start; index < this->m_nString; index++ )
		{
			if ( this->m_szString[index] == find )
			{
				next = index; // Save new position and finish.
				break; // End FOR loop!
			}
		}
	}
	return next;
}

size_t CString::FindLast( const TCHAR find )
{
	size_t result = -1L;

	if ( this->m_szString != NULL )
	{
		// Try to (reverse) find last character.
		for ( size_t index=this->m_nString; index > 0; index-- )
		{
			if ( this->m_szString[index] == find )
			{
				result = index;
				break; // End FOR loop!
			}
		}
	}
	return result;
}

TCHAR CString::GetAt( size_t index )
{
	TCHAR result = _T('\0');

	if ( this->m_szString != NULL )
	{
		if ( 0 <= index && index < this->m_nString )
		{
			result = this->m_szString[index];
		}
	}
	return result;
}

TCHAR* CString::CreateBuffer( size_t count )
{
	this->ReleaseBuffer();

	this->m_szBuffer = new TCHAR[ count + 1 ];

	if ( this->m_szBuffer != NULL )
	{
		for ( size_t index=0; index < count + 1; index++ )
		{
			this->m_szBuffer[index] = _T('\0');
		}
	}
	return this->m_szBuffer;
}

bool CString::CopyBuffer()
{
	return this->Create( this->m_szBuffer );
}

void CString::ReleaseBuffer()
{
	delete this->m_szBuffer;
	this->m_szBuffer = NULL;
}

bool CString::Create( size_t count )
{
	bool success = false;

	this->Destroy();

	this->m_nString  = count;
	this->m_szString = new TCHAR[ this->m_nString + 1 ];

	if ( this->m_szString != NULL )
	{
		// Intentionally, function memset() is not called!
		for ( size_t index=0; index < this->m_nString + 1; index++ )
		{
			this->m_szString[index] = _T('\0');
		}
		success = true;
	}
	else
	{
		this->m_nString = 0;
	}
	return success;
}

bool CString::Create( const TCHAR* szString )
{
	bool success = false;

	this->Destroy();

	if ( szString != NULL )
	{
		this->m_nString  = _tcslen( szString );
		this->m_szString = new TCHAR[ this->m_nString + 1 ];

		if ( this->m_szString != NULL )
		{
			if ( 0 == _tcscpy_s( this->m_szString, this->m_nString + 1, szString ) )
			{
				success = true;
			}
		}
		else
		{
			this->m_nString = 0;
		}
	}
	return success;
}

void CString::Destroy()
{
	delete this->m_szString;

	this->m_nString  = 0;
	this->m_szString = NULL;
}
