/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include <tchar.h>

namespace plexdata
{
	namespace string
	{
		class CString
		{
			size_t m_nString;
			TCHAR* m_szString;
			TCHAR* m_szBuffer;

		public:

			CString();

			CString( const TCHAR* szString );

			CString( CString& refString );

			CString( CString* pString );

			virtual ~CString();

		public:

			inline const TCHAR* String()
			{
				return this->m_szString;
			}

			inline size_t Length()
			{
				return this->m_nString;
			}

			inline bool IsEmpty()
			{
				// The string is empty if its length is zero.
				if ( 0 == this->m_nString )
				{
					return true;
				}
				// The string is empty too if it only contains the zero termination.
				else if ( 1 == this->m_nString && this->m_szString[0] == _T('\0') )
				{
					return true;
				}
				else
				{
					return false;
				}
			}

		public:

			inline CString& operator += ( const TCHAR* szString )
			{
				this->Append( szString );
				return *this;
			}

			inline CString& operator += ( CString& refString )
			{
				this->Append( refString );
				return *this;
			}

			inline CString& operator += ( CString* pString )
			{
				this->Append( pString );
				return *this;
			}

			inline CString& operator = ( const TCHAR* szString )
			{
				this->Copy( szString );
				return *this;
			}

			inline CString& operator = ( CString& refString )
			{
				this->Copy( refString );
				return *this;
			}

			inline CString& operator = ( CString* pString )
			{
				this->Copy( pString );
				return *this;
			}

			inline bool operator == ( const TCHAR* szString )
			{
				return this->Equals( szString, false );
			}

			inline bool operator == ( CString& refString )
			{
				return this->Equals( refString, false );
			}

			inline bool operator == ( CString* pString )
			{
				return this->Equals( pString, false );
			}

			inline bool operator != ( const TCHAR* szString )
			{
				return !this->Equals( szString, false );
			}

			inline bool operator != ( CString& refString )
			{
				return !this->Equals( refString, false );
			}

			inline bool operator != ( CString* pString )
			{
				return !this->Equals( pString, false );
			}

		public:

			bool Append( const TCHAR* szString );

			bool Append( CString& refString );

			bool Append( CString* pString );

			bool Copy( const TCHAR* szString );

			bool Copy( CString& refString );

			bool Copy( CString* pString );

			bool Equals( const TCHAR* szString, bool ignore=true );

			bool Equals( CString& refString, bool ignore=true );

			bool Equals( CString* pString, bool ignore=true );

			bool Format( const TCHAR* szFormat, ... );

			CString Substring( size_t first, size_t last=-1L );

			CString Trim();

			CString LeftTrim();

			CString RightTrim();

			size_t FindFirst( const TCHAR find );

			size_t FindNext( const TCHAR find, size_t start );

			size_t FindLast( const TCHAR find );

			TCHAR GetAt( size_t index );

			TCHAR* CreateBuffer( size_t count );

			bool CopyBuffer();

			void ReleaseBuffer();

		private:

			bool Create( size_t count );

			bool Create( const TCHAR* szString );

			void Destroy();
		};
	}
}