/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "stdafx.h"
#include <stdlib.h>

int _tmain(int argc, _TCHAR* argv[])
{
	// Open logger at a console window.
	HANDLE hLogger = ::LogAttach
		( LOGTYPE_CONSOLE
		, TRUE
		, FALSE 
		, TRUE
		, _T("Log API Tester")
		);

	if ( hLogger == NULL )
	{
		_tprintf( _T("TEST: Open Logger failed (error=%d).\n"), ::GetLastError() );
	}

	// Log some messages.
	_tprintf( _T("TEST: a message.\n") );
	::LogMessage( hLogger, _T("Hello, World!\n") );

	_tprintf( _T("TEST: an empty message.\n") );
	::LogMessage( hLogger, _T("") );

	// Test special (problematic) characters
	::LogMessage( hLogger, _T("%") );
	::LogMessage( hLogger, _T("%%") );
	::LogMessage( hLogger, _T("%%%") );

	// Resize logger and log large messages, warnings and errors.
	::LogResize( hLogger, 1000, 500 );
	::LogMessage( hLogger, _T("Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World!") );
	::LogWarning( hLogger, _T("Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World!") );
	::LogError( hLogger, _T("Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World! Hello, World!") );

	// Log a memory dump.
	wchar_t* pBuffer = new wchar_t[150];
	::LogDump( hLogger, pBuffer, 150, true );
	delete pBuffer;

	// Hardcore large message test including color switching.
	TCHAR dummy [500000]=_T("");
	int i=0;
	while (  i < 1000 )
	{
		Sleep(100);

		_tcscat_s( dummy, ARRAYSIZE(dummy), _T("Hello World") );

		if ( i % 9 == 0 )
			_tcscat_s( dummy, ARRAYSIZE(dummy), _T("\n") );

		::LogError( hLogger, dummy );
		_tprintf( _T("        "));
		::LogMessage( hLogger, dummy );
		_tprintf( _T("        "));
		::LogWarning( hLogger, dummy );

		_tprintf( _T("        "));
		int nBuffer = i;
		wchar_t* pBuffer = new wchar_t[nBuffer];
		::LogDump( hLogger, pBuffer, nBuffer, true );
		delete pBuffer;
		i++;

		::LogColors( hLogger, i%2 );
	}

	// Close logger.
	::LogDetach( hLogger );

	system("pause");

	return 0;
}

